package controllers.psec;


import play.Logger;
import play.data.Form;
import play.i18n.Messages;
import play.mvc.Controller;
import play.mvc.Result;
import de.ppi.psec.permissions.IsPublic;
import de.ppi.psec.services.AuthenticationServiceImpl;
import views.html.*;

@IsPublic
public class AuthController extends Controller {

    public static final String FLASH_PSEC_URL = "psecUrl";

    
    public static class Login {       
        public String loginname;
        public String password;
        public boolean remember;
        public String javaScriptOn;
        
        public String validate() {
            try {
                if(!AuthenticationServiceImpl.getService().login(loginname, 
                        password, remember, "Y".equals(javaScriptOn))) {
                    return Messages.get("psec.login.failed");
                }
            } catch (RuntimeException rE) {
                Logger.error("Problem during validate Login", rE);
                throw rE;
            }
            return null;
        }
        
    }
    
    
    public static Result login() {
        final Form<Login> loginForm = form(Login.class);
        return ok(login.render(loginForm));
    }

    public static Result signIn() {
        final Form<Login> loginForm = form(Login.class).bindFromRequest();
        if(loginForm.hasErrors()) {
            return badRequest(login.render(loginForm));
        } else {            
            return redirectToInteruptedURL();
        }
    }
    
    public static Result logout() {
        AuthenticationServiceImpl.getService().logout();
        return redirectToInteruptedURL();
    }

    private static Result redirectToInteruptedURL() {
        String url = session().get(FLASH_PSEC_URL);
        
        if(url == null) {
            url = "/";
        }
        return redirect(url);
    }


}
