// (C) 2009 PPI AG
package de.ppi.psec.permissions;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.security.Permission;
import java.util.HashMap;



/**
 * Factory which deliver an existing instance of a permission if exists,
 * otherwise it will create a new one.
 * @author niels
 */
public class PsecPermissionFactory {

    private static final HashMap <String, Permission> PERMISSIONS =
            new HashMap <String, Permission>();

    public static final synchronized Permission getPermission(String className,
            String target, String action) {
        final String key = className + "$%&@" + target + "$%&@" + action;
        if (!PERMISSIONS.containsKey(key)) {
            Permission perm = null;
            try {
                perm = createPermission(className, target, action);
            } catch (Exception e) {
                throw new CouldntCreatePermissionException(className, e);
            }
            PERMISSIONS.put(key, perm);
        }
        return PERMISSIONS.get(key);
    }

    @SuppressWarnings("rawtypes")
    private static final Permission createPermission(String className,
            String target, String action) throws ClassNotFoundException,
            IllegalArgumentException, InstantiationException,
            IllegalAccessException, InvocationTargetException {
        if (className.equals(PsecBasicPermission.class.getName())) {
            return new PsecBasicPermission(target, action);
        }
        final Class clazz = PsecPermissionFactory.class.
                getClassLoader().loadClass(className);
        // Check the available constructors. If any take two
        // string parameters, we use that one and pass in the
        // target and actions string. Otherwise we try a single
        // parameter constructor and pass in just the target.
        Constructor preferredConstructor = null;
        Constructor fallbackConstructor = null;
        Constructor standardConstructor = null;
        Constructor[] constructors = clazz.getDeclaredConstructors();
        for (Constructor constructor : constructors) {
            switch(constructor.getParameterTypes().length) {
            case 2:
                if (constructor.getParameterTypes()[0].equals(String.class) &&
                        constructor.getParameterTypes()[1].equals(String.class)) {
                    preferredConstructor = constructor;
                }
                break;
            case 1:
                if (constructor.getParameterTypes()[0].equals(String.class)) {
                    fallbackConstructor = constructor;
                }
                break;
            case 0:
                standardConstructor = constructor;
            }
        }
        if (preferredConstructor != null) {
            return (Permission) preferredConstructor.newInstance(target, action);
        } else if (fallbackConstructor != null) {
            return (Permission) fallbackConstructor.newInstance(target);
        } else if (standardConstructor != null) {
            return (Permission) standardConstructor.newInstance();
        } else {
            throw new CouldntCreatePermissionException(
                    "No constructor found for " + className);
        }

    }

    public static class CouldntCreatePermissionException extends RuntimeException {

        public CouldntCreatePermissionException(String message, Throwable cause) {
            super(message, cause);
        }

        public CouldntCreatePermissionException(String message) {
            super(message);
        }
    }


}
