package de.ppi.psec.services;

import java.security.Permission;
import java.util.List;


public interface AuthenticationService {

    /**
     * Check if the user is authenticated.
     * @param allowRememberMe true if it be OK if the user is only
     * authenticated by a rememberMeCookie.
     * @return true if the user is authenticated.
     */
    public boolean isAuthenticate(boolean allowRememberMe);
    
    
    /**
     * Checks if the user can be remembered by a cookie.
     * @return true, if the user can be remembered.
     */
    public boolean canRemembered();

    /**
     * Make a login.
     * @param loginname Loginnamne
     * @param password the password.
     * @param rememberMe true if a user should be remembered.
     * @param javaScriptOn true if java-script is active.
     * @return true if the user is successfuly authenticated.
     */
    public boolean login(String loginname, String password, boolean rememberMe,
            boolean javaScriptOn);

    
    /**
     * Logut the User.
     */
    public void logout();

    /**
     * Returns the username of the current user which has log in.
     */
    public String getLoginname();

    /**
     * Returns the id of the current user which has log in.
     */
    public String getUserid();

    /**
     * Returns the lastname of the current user which has log in.
     */
    public String getLastname();

    /**
     * Returns the firstname of the current user which has log in.
     */
    public String getFirstname();

    /**
     * Returns the rolenames of the current user which has log in.
     */
    public List<String> getRoles();

    /**
     * Returns the direct permissions of the current user which has log in.
     */
    public List<Permission> getPermissions();

    /**
     *  Remove all cached values.     
     *  The cache contains all information, which hangs directly at the user,
     *  like name, roles and direct permission.
     */
    public void clearCache();

}