package de.ppi.psec.services;

import java.security.Permission;

import de.ppi.psec.permissions.PsecBasicPermission;

public interface AuthorizationService {

    /**
     * clear the cached permission.
     * @param reason the reason why the cache should be cleared.
     */
    public void clearPermissionCache(String reason);

    /**
     * Check if the current user has the given role.
     * @param roleName the name of the role.
     * @return true if the current user has the given role.
     */
    public boolean hasRole(String roleName);

    /**
     * Check if the current user has the given permission.
     * @param requiredPermission the name of the permission.
     * @return true if the current user has the given permission.
     */
    public boolean hasPermission(Permission requiredPermission);

    /**
     * Check if the current user has the given permission.
     * @param targetName defines the target of a {@linkplain PsecBasicPermission}
     * @param actionName defines the action of a {@linkplain PsecBasicPermission}
     * @return true if the current user has the given permission.
     */
    public boolean hasPermission(String targetName, String actionName);

}