/**
 * 
 */
package de.ppi.util;

import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.KeyGenerator;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.codec.binary.Hex;

import play.Logger;
import play.Play;

/**
 * Helper class tp encrypt or decrypt a String.
 * @author niels
 *
 */
public class Crypto {

	public static void blowfish() throws NoSuchAlgorithmException, NoSuchPaddingException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException, UnsupportedEncodingException {
		String text = "java2s";
	
	    KeyGenerator keyGenerator = KeyGenerator.getInstance("Blowfish");
	    keyGenerator.init(128);
	
	    Key key = keyGenerator.generateKey();
	
	    Cipher cipher = Cipher.getInstance("Blowfish/ECB/PKCS5Padding");
	    cipher.init(Cipher.ENCRYPT_MODE, key);
	
	    byte[] ciphertext = cipher.doFinal(text.getBytes("UTF8"));
	
	    for (int i = 0; i < ciphertext.length; i++) {
	      System.out.print(ciphertext[i] + " ");
	    }
	    cipher.init(Cipher.DECRYPT_MODE, key);
	    byte[] decryptedText = cipher.doFinal(ciphertext);
	
	    System.out.println(new String(decryptedText, "UTF8"));
	}
    /**
     * Encrypt a String with the AES encryption standard using the application secret
     * @param value The String to encrypt
     * @return An hexadecimal encrypted string
     */
    public static String encryptAES(String value) {
        return encryptAES(value, Play.application().configuration().
        		getString("application.secret").substring(0, 16));
    }

    /**
     * Encrypt a String with the AES encryption standard. Private key must have a length of 16 bytes
     * @param value The String to encrypt
     * @param privateKey The key used to encrypt
     * @return An hexadecimal encrypted string
     * @throws InvalidKeyException 
     */
    public static String encryptAES(String value, String privateKey)  {
        try {
            byte[] raw = privateKey.getBytes();
            SecretKeySpec skeySpec = new SecretKeySpec(raw, "AES");
            Cipher cipher = Cipher.getInstance("AES");
            cipher.init(Cipher.ENCRYPT_MODE, skeySpec);
            return String.valueOf(Hex.encodeHex(cipher.doFinal(value.getBytes())));
        } catch (Exception e) {
            Logger.of(Crypto.class).error(e.getLocalizedMessage(), e);
            throw new EncryptionException(e);
        }
    }

    /**
     * Decrypt a String with the AES encryption standard using the application secret
     * @param value An hexadecimal encrypted string
     * @return The decrypted String
     */
    public static String decryptAES(String value) {
        return decryptAES(value, Play.application().configuration().
        		getString("application.secret").substring(0, 16));
    }

    /**
     * Decrypt a String with the AES encryption standard. Private key must have a length of 16 bytes
     * @param value An hexadecimal encrypted string
     * @param privateKey The key used to encrypt
     * @return The decrypted String
     */
    public static String decryptAES(String value, String privateKey) {
        try {
            byte[] raw = privateKey.getBytes();
            SecretKeySpec skeySpec = new SecretKeySpec(raw, "AES");
            Cipher cipher = Cipher.getInstance("AES");
            cipher.init(Cipher.DECRYPT_MODE, skeySpec);
            return new String(cipher.doFinal(Hex.decodeHex(value.toCharArray())));
        } catch (Exception e) {
            Logger.of(Crypto.class).error(e.getLocalizedMessage(), e);
            throw new EncryptionException(e);
        }
    }
    
    
    /**
     * This exception indicates that's sometghing goes wrong when try to encrypt
     * or decrypt a string.
     *
     */
    public static final class EncryptionException extends RuntimeException {

        public EncryptionException(String message, Throwable cause) {
            super(message, cause);
        }

        public EncryptionException(Throwable cause) {
            super(cause);
        }
        
    }

}
