package models.psec;

import java.security.Permission;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.PrePersist;
import javax.persistence.PreUpdate;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import play.db.ebean.Model;
import de.ppi.psec.permissions.PsecBasicPermission;
import de.ppi.psec.permissions.PsecPermissionFactory;

@Entity
public class PsecPermission extends Model {

    @Id
    @GeneratedValue
    private Long id;
	
    @Column(unique=true, nullable=false)
  //TODO niels @Unique
    private String name;

    @Column(nullable=false)
  //TODO niels @Unique("target, actions")
    //TODO niels @CheckWith(value=ValidPermissionCheck.class)
    private String type = PsecBasicPermission.class.getName();

    @Column(nullable=false)
    private String target;

    @Column(nullable=false)
    private String actions;

//    @ManyToMany(mappedBy="permissions")
//    public List<PsecRole> roles;
    
    @Temporal(TemporalType.TIMESTAMP)
    private Date lastUpdate;
    
    private boolean autoUpdatesForbidden = false;


    public static PsecPermission 
            createFromBasicPermission(String name, PsecBasicPermission perm) {
        final PsecPermission result = new PsecPermission();
        result.name = name;
        result.type = perm.getClass().getName();
        result.target = perm.getTarget();
        result.actions = perm.getActions();
        return result;
    }
    
    public Permission convertToPermission() {
        return PsecPermissionFactory.getPermission(type, target, actions);
    }

    @Override
    public String toString()  {
        return name;
    }

    @PreUpdate
    @PrePersist
    protected void onUpdate() {
        lastUpdate = new Date();
    }

    
    

    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = super.hashCode();
        result = prime * result + ((actions == null) ? 0 : actions.hashCode());
        result = prime * result + (autoUpdatesForbidden ? 1231 : 1237);
        result = prime * result + ((name == null) ? 0 : name.hashCode());
        result = prime * result + ((target == null) ? 0 : target.hashCode());
        result = prime * result + ((type == null) ? 0 : type.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (!super.equals(obj))
            return false;
        if (getClass() != obj.getClass())
            return false;
        PsecPermission other = (PsecPermission) obj;
        if (actions == null) {
            if (other.actions != null)
                return false;
        } else if (!actions.equals(other.actions))
            return false;
        if (autoUpdatesForbidden != other.autoUpdatesForbidden)
            return false;
        if (name == null) {
            if (other.name != null)
                return false;
        } else if (!name.equals(other.name))
            return false;
        if (target == null) {
            if (other.target != null)
                return false;
        } else if (!target.equals(other.target))
            return false;
        if (type == null) {
            if (other.type != null)
                return false;
        } else if (!type.equals(other.type))
            return false;
        return true;
    }

    public Long getId() {
        return id;
    }

    
    public void setId(Long id) {
        this.id = id;
    }

    
    public String getName() {
        return name;
    }

    
    public void setName(String name) {
        this.name = name;
    }

    
    public String getType() {
        return type;
    }

    
    public void setType(String type) {
        this.type = type;
    }

    
    public String getTarget() {
        return target;
    }

    
    public void setTarget(String target) {
        this.target = target;
    }

    
    public String getActions() {
        return actions;
    }

    
    public void setActions(String actions) {
        this.actions = actions;
    }

    
    public Date getLastUpdate() {
        return lastUpdate;
    }

    
    public void setLastUpdate(Date lastUpdate) {
        this.lastUpdate = lastUpdate;
    }

    /**
     * Return true if updates at application updates are forbidden.
     * @return true if updates at application updates are forbidden.
     */
    public boolean getAutoUpdatesForbidden() {
        return autoUpdatesForbidden;
    }

    /**
     * Set the flag, that updates at application updates are forbidden.
     * @param autoUpdatesForbidden flag, that updates at application updates are forbidden.
     */
    public void setAutoUpdatesForbidden(boolean autoUpdatesForbidden) {
        this.autoUpdatesForbidden = autoUpdatesForbidden;
    }


    /**
     * Check to prove is the given classname exists.
     *
     */
//    private static class ValidPermissionCheck extends Check {
//
//        @Override
//        public boolean isSatisfied(Object validatedObject, Object value) {
//            if (value != null) {
//                try {
//                    ((PsecPermission)validatedObject).convertToPermission();
//                } catch (IllegalStateException e) {
//                    setMessage("psec.illegal.permission", value.toString());
//                    return false;
//                }
//            }
//            return true;
//        }
//
//    }
    
       
}
