package models.psec;

import java.security.Permission;
import java.util.Date;
import java.util.List;


public interface User {

    /**
     * The technical database-id of the user.
     * @return the technical database-id of the user
     */
    public Long getId();
    
    /**
     * All roles in from of the name the user has.
     * @return all roles the user has.
     */
    public List <String> getRoles();

    
    /**
     * Add a role to the user.
     * @param permsecRole the role which should be added.
     */
    public void addToRoles(PsecRole permsecRole);

    /**
     * All permissions the user has.
     * @return all permissions the user has.
     */
    public List <Permission> getPermissions();


    /**
     * Get the name of the user which is used for login.
     * @return the name of the user which is used for login.
     */
    public String getLoginname();

    /**
     * Set the name of the user which is used for login.
     * @param loginname the name of the user which is used for login.
     */
    public void setLoginname(String loginname);

    
    /**
     * Returns the emailaddress.
     * @return the emailaddress.
     */
    public String getEmailAddress();

    /**
     * Set the emailaddress.
     * @param emailAddress the emailaddress.
     */
    public void setEmailAddress(String emailAddress);

    /**
     * Returns the lastname.
     * @return the lastname.
     */
    public String getLastname();

    /**
     * Set the lastname.
     * @param lastname the lastname.
     */
    public void setLastname(String lastname);

    /**
     * Returns the firstname.
     * @return the firstname.
     */
    public String getFirstname();

    /**
     * Set the firstname.
     * @param firstname the firstname.
     */
    public void setFirstname(String firstname);

    /**
     * Returns the description of the user.
     * @return the description of the user.
     */
    public String getDescription();

    /**
     * Set the description of the user.
     * @param description the description of the user.
     */
    public void setDescription(String description);

    /**
     * Returns the number of logins which fails since last successful login.
     * @return the number of logins which fails since last successful login.
     */
    public int getLoginFailedCount();

    /**
     * Set the number of logins which fails since last successful login.
     * @param loginFailedCount the number of logins which fails since last successful login.
     */
    public void setLoginFailedCount(int loginFailedCount);

    /**
     * Returns true if the user is enabled.
     * @return true if the user is enabled.
     */
    public boolean isEnabled();

    /**
     * Set teh enabled state of the user.
     * @param enabled true if the user is enabled.
     */
    public void setEnabled(boolean enabled);
    
    /**
     * Returns the date at which the registration expired.
     * @return the date at which the registration expired.
     */
    public Date getExpiredDate();

    /**
     * Set the date at which the registration expired.
     * @param expiredDate the date at which the registration expired.
     */
    public void setExpiredDate(Date expiredDate);

    /**
     * Returns the password-hash.
     * @return the password-hash.
     */
    public String getPasswordHash();


    /**
     * Set the password.
     * @param password the password.
     */
    public void setPassword(String newPassword);
    
    /**
     * Check if the given password is valid.
     */
    public boolean checkPassword(String givenPassword);

  
}