/**
 *
 */
package de.ppi.psec.services;

import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

import java.security.Permission;
import java.util.ArrayList;
import java.util.List;

import models.psec.Role;

import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.ppi.psec.permissions.PsecBasicPermission;

/**
 * @author niels
 *
 */
public class AuthorizationServiceTest {

    AuthorizationService service;

    AuthenticationService authServiceMock = mock(AuthenticationService.class);

    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        
    }


    @Before
    public void setUp() throws Exception {        
        service = new AuthorizationServiceImpl(authServiceMock, new RolePersistanceServiceEbean(), true);
    }

    /**
     * Test method for {@link de.ppi.psec.services.AuthorizationService#hasRole(java.lang.String)}.
     */
    @Test
    public void testHasRole() {
        List<String> roles = new ArrayList<String>();
        roles.add("TESTER");
        roles.add("DEVELOPER");
        when(authServiceMock.getRoles()).thenReturn(roles);
        assertFalse(service.hasRole("foo"));
        assertFalse(service.hasRole("TEST"));
        assertFalse(service.hasRole("Tester"));
        assertTrue(service.hasRole("TESTER"));
        assertTrue(service.hasRole("DEVELOPER"));
    }

    /**
     * Test method for {@link de.ppi.psec.services.AuthorizationService#hasPermission(java.security.Permission)}.
     */
    @Test
    public void testHasPermissionPermission() {
        List<Permission> permissions = new ArrayList<Permission>();
        permissions.add(new PsecBasicPermission("foo", "bar, git"));
        permissions.add(new PsecBasicPermission("good", "bzr, hg"));
        when(authServiceMock.getRoles()).thenReturn(new ArrayList <String>());        
        when(authServiceMock.getPermissions()).thenReturn(permissions);
        assertFalse(service.hasPermission(new PsecBasicPermission("*", "*")));
        assertFalse(service.hasPermission(new PsecBasicPermission("foo", "*")));
        assertFalse(service.hasPermission(new PsecBasicPermission("*", "bzr, hg")));
        assertTrue(service.hasPermission(new PsecBasicPermission("foo", "bar")));
        assertTrue(service.hasPermission(new PsecBasicPermission("foo", "git")));
        permissions.add(new PsecBasicPermission("*", "*"));
        assertTrue(service.hasPermission(new PsecBasicPermission("*", "*")));
        assertTrue(service.hasPermission(new PsecBasicPermission("foo", "*")));
        assertTrue(service.hasPermission(new PsecBasicPermission("*", "bzr, hg")));
    }

    /**
     * Test method for {@link de.ppi.psec.services.AuthorizationService#hasPermission(java.lang.String, java.lang.String)}.
     */
    @Test
    public void testHasPermissionStringString() {
        List<Permission> permissions = new ArrayList<Permission>();
        permissions.add(new PsecBasicPermission("foo", "bar, git"));
        permissions.add(new PsecBasicPermission("good", "bzr, hg"));
        when(authServiceMock.getRoles()).thenReturn(new ArrayList <String>());        
        when(authServiceMock.getPermissions()).thenReturn(permissions);
        assertFalse(service.hasPermission("*", "*"));
        assertFalse(service.hasPermission("foo", "*"));
        assertFalse(service.hasPermission("*", "bzr, hg"));
        assertTrue(service.hasPermission("foo", "bar"));
        assertTrue(service.hasPermission("foo", "git"));
        permissions.add(new PsecBasicPermission("*", "*"));
        assertTrue(service.hasPermission("*", "*"));
        assertTrue(service.hasPermission("foo", "*"));
        assertTrue(service.hasPermission("*", "bzr, hg"));
    }
    
    /**
     * Test method for {@link de.ppi.psec.services.AuthorizationService#hasPermission(java.lang.String, java.lang.String)}.
     */
    @Test
    public void testWithRoleHasPermissionStringString() {
        final RolePersistanceService rolePersServiceMock = mock(RolePersistanceService.class);
        final AuthenticationService authenticationServiceMock = mock(AuthenticationService.class);
        final AuthorizationService authService = new AuthorizationServiceImpl(
                authenticationServiceMock, rolePersServiceMock, true);
        final Role roleMock = mock(Role.class);
        final List <String> roles = new ArrayList <String>();
        roles.add("ROLE1");
        when(authenticationServiceMock.getRoles()).thenReturn(roles);
        final List<Permission> permissions = new ArrayList<Permission>();
        permissions.add(new PsecBasicPermission("foo", "bar, git"));
        permissions.add(new PsecBasicPermission("good", "bzr, hg"));
        when(roleMock.getPermissions()).thenReturn(permissions);
        when(rolePersServiceMock.findRoleByName("ROLE1")).thenReturn(roleMock);
       
        List<Permission> directPermissions = new ArrayList<Permission>();
        directPermissions.add(new PsecBasicPermission("unknown", "unknown")); 
        when(authenticationServiceMock.getPermissions()).thenReturn(directPermissions);
        
        assertFalse(authService.hasPermission("*", "*"));
        assertFalse(authService.hasPermission("foo", "*"));
        assertFalse(authService.hasPermission("*", "bzr, hg"));
        assertTrue(authService.hasPermission("foo", "bar"));
        assertTrue(authService.hasPermission("foo", "git"));
    }
    
}
