import java.lang.reflect.Method;

import com.avaje.ebean.Ebean;

import play.Application;
import play.GlobalSettings;
import play.mvc.Action;
import play.mvc.Http.Request;
import controllers.psec.routes;
import de.ppi.psec.helper.EbeanInitDBHelper;
import de.ppi.psec.helper.PsecAction;



/**
 * Startup and security hooks.
 * @author niels
 *
 */
public class Global extends GlobalSettings {
    private static final boolean ENABLED = true;


    @Override
    public void onStart(Application arg0) {
        final EbeanInitDBHelper initDB = new EbeanInitDBHelper();
        Ebean.beginTransaction();
        //Create Permissions.
        initDB.createOrUpdatePermission("root", "*", "*");
        initDB.createOrUpdatePermission("PsecUser.All", "controllers.PsecUsers", "attachment, blank, create, delete, index, list, save, show");
        initDB.createOrUpdatePermission("PsecRoles.All", "controllers.PsecRoles", "*");
        initDB.createOrUpdatePermission("PsecPermissions.All", "controllers.PsecPermissions", "*");
        initDB.createOrUpdatePermission("ProtectedA.1", "controllers.ProtectedA", "perm1");
        initDB.createOrUpdatePermission("ProtectedB.2", "controllers.ProtectedB", "perm2");
        initDB.createOrUpdatePermission("example1", "target1", "action1.3, action1.2");
        initDB.createOrUpdatePermission("example2", "target2", "action2.2, action2.2");
        initDB.createOrUpdatePermission("example3", "target3", "action3.1, action3.2");
        
        //Create Roles
        initDB.createOrUpdateRole("exampleRole", false, "example1", "example2");
        initDB.createOrUpdateRole("adminRole", false, "PsecUser.All", "PsecRoles.All", "PsecPermissions.All");
        initDB.createOrUpdateRole("aRole", false, "ProtectedA.1");
        initDB.createOrUpdateRole("bRole", false, "ProtectedB.2");
        
        //Create Users
        initDB.createOrUpdateUser("root", null, "", "", "", "Geheim", true, "root", "");
        initDB.createOrUpdateUser("admin", null, "", "", "", "Secret", true, null, "adminRole");
        initDB.createOrUpdateUser("userA", null, "", "", "", "123123", true, null, "aRole");
        initDB.createOrUpdateUser("userB", null, "", "", "", "123123", true, null, "bRole");
        initDB.createOrUpdateUser("test", null, "", "", "", "Secret", true, null, "exampleRole");
        Ebean.commitTransaction();
        super.onStart(arg0);
    }




    @SuppressWarnings("rawtypes")
    @Override
    public Action onRequest(Request request, Method actionMethod) {
        return new PsecAction(ENABLED, routes.AuthController.login().url(), actionMethod);
    }

}
